# -*- coding: utf-8 -*-

from functools import wraps

from dagny.renderer import Renderer
from dagny.resource import Resource
from dagny.utils import resource_name


class Action(object):
    
    """
    A descriptor for wrapping an action method.
    
        >>> action = Action
        >>> class X(Resource):
        ...     @action
        ...     def show(self):
        ...         self.attr1 = 'a'
    
    Appears as an `Action` on the class (and other objects):
    
        >>> X.show  # doctest: +ELLIPSIS
        <Action '#show' at 0x...>
        >>> X.show.render  # doctest: +ELLIPSIS
        <BoundRenderer on <Action '#show' at 0x...>>
    
    Appears as a `BoundAction` on an instance:
    
        >>> x = X._new(object())
        >>> x.show  # doctest: +ELLIPSIS
        <BoundAction 'X#show' at 0x...>
        >>> x.show.render  # doctest: +ELLIPSIS
        <bound method BoundAction.render of <BoundAction 'X#show' at 0x...>>
    
    ## Actions and Rendering
    
    The API for `Action` instances has been fine-tuned to allow an easy
    interface with the renderer system. When accessed from inside the class
    definition, `show.render` is a `BoundRenderer` instance, so you're just
    using the standard decorator syntax for defining new renderer backends:
    
        class User(Resource):
            
            @action
            def show(self, username):
                self.user = get_object_or_404(User, username=username)
            
            @show.render.json
            def show(self):
                return JSONResponse(self.user.as_dict())
            
            # You can also un-define renderer backends for a single action:
            del show.render['html']
            
            # Or assign generic backends for a particular action:
            show.render['html'] = my_generic_html_backend
    
    When accessed via a resource *instance*, `show.render` will be the
    `render()` method of a `BoundAction`, and calling it will invoke the full
    rendering process for that particular action, on the current resource. This
    is very useful when handling forms; for example:
    
        class User(Resource):
            
            @action
            def edit(self, username):
                self.user = get_object_or_404(User, username=username)
                self.form = UserForm(instance=self.user)
                # Here the default HTML renderer will kick in; you should
                # display the edit user form in a "user/edit.html" template.
            
            @action
            def update(self, username):
                self.user = get_object_or_404(User, username=username)
                self.form = UserForm(instance=self.user)
                if self.form.is_valid():
                    self.form.save()
                    # Returns a response, action ends here.
                    return redirect(self.user)
                
                # Applies the `edit` renderer to *this* request, thus rendering
                # the "user/edit.html" template but with this resource (and
                # hence this `UserForm` instance, which contains errors).
                response = self.edit.render()
                response.status_code = 403  # Forbidden
                return response
    
    It makes sense to write the `user/edit.html` template so that it renders
    forms dynamically; this means the filled-in fields and error messages will
    propagate automatically, without any extra work on your part.
    """
    
    # Global renderer to allow definition of generic renderer backends.
    RENDERER = Renderer()
    
    @staticmethod
    def deco(decorator):
        
        """
        Static method to wrap a typical view decorator as an action decorator.
        
        Usage is relatively simple, but remember that `@action.deco()` must come
        *below* `@action`:
        
            class User(Resource):
                
                @action
                @action.deco(auth_required)
                def edit(self, username):
                    ...
        
        This will wrap the decorator so that *it* sees a function with a
        signature of `view(request, *args, **kwargs)`; this function is an
        adapter which will then call the action appropriately.
        
            >>> def decorator(view):
            ...     def wrapper(request, *args, **kwargs):
            ...         print "WRAPPER (In):"
            ...         print "  ", repr(request), args, kwargs
            ...         
            ...         request = "ModifiedRequest"
            ...         args = ("another_user",)
            ...         kwargs.update(authenticated=True)
            ...
            ...         print "WRAPPER (Out):"
            ...         print "  ", repr(request), args, kwargs
            ...         
            ...         return view(request, *args, **kwargs)
            ...     return wrapper
            
            >>> def view(self, username):
            ...     print "VIEW:"
            ...     print "  ", repr(self.request), self.args, self.params
            
            >>> resource = type('Resource', (object,), {})()
            >>> resource.request = "Request"
            >>> resource.args = ("some_user",)
            >>> resource.params = {}
            
            >>> view(resource, "some_user")
            VIEW:
               'Request' ('some_user',) {}
            
            >>> Action.deco(decorator)(view)(resource, "some_user")
            WRAPPER (In):
               'Request' ('some_user',) {}
            WRAPPER (Out):
               'ModifiedRequest' ('another_user',) {'authenticated': True}
            VIEW:
               'ModifiedRequest' ('another_user',) {'authenticated': True}
        
        """
        
        @wraps(decorator)
        def deco_wrapper(action_func):
            @wraps(action_func)
            def action_wrapper(self, *args):
                @wraps(action_func)
                def adapter(request, *adapter_args, **params):
                    self.request = request
                    self.args = adapter_args
                    self.params = params
                    return action_func(self, *self.args)
                return decorator(adapter)(self.request, *args, **self.params)
            return action_wrapper
        return deco_wrapper
    
    def __init__(self, method):
        self.method = method
        self.name = method.__name__
        self.render = self.RENDERER._bind(self)
    
    def __repr__(self):
        return "<Action '#%s' at 0x%x>" % (self.name, id(self))
    
    def __get__(self, resource, resource_cls):
        if isinstance(resource, Resource):
            return BoundAction(self, resource, resource_cls)
        return self


class BoundAction(object):
    
    """An action which has been bound to a specific resource instance."""
    
    def __init__(self, action, resource, resource_cls):
        self.action = action
        self.resource = resource
        self.resource_cls = resource_cls
        self.resource_name = resource_name(resource_cls)
    
    def __repr__(self):
        return "<BoundAction '%s#%s' at 0x%x>" % (self.resource_name, self.action.name, id(self))
    
    def __call__(self):
        response = self.action.method(self.resource, *self.resource.args)
        if response:
            return response
        return self.render()
    
    def render(self):
        return self.action.render(self.resource)
    
    @property
    def name(self):
        return self.action.name



# -*- coding: utf-8 -*-

from dagny.action import Action
from dagny.utils import camel_to_underscore, resource_name


@Action.RENDERER.html
def render_html(action, resource):
    
    """
    Render an appropriate HTML response for an action.
    
    This is a generic renderer backend which produces HTML responses. It uses
    the name of the resource and current action to generate a template name,
    then renders the template with a `RequestContext`.
    
    To retrieve the template name, the resource name is first turned from
    CamelCase to lowercase_underscore_separated; if the class name ends in
    `Resource`, this is first removed from the end. For example:
    
        User => user
        UserResource => user
        NameXYZ => name_xyz
        XYZName => xyz_name
    
    You can optionally define a template path prefix on your `Resource` like so:
    
        class User(Resource):
            template_path_prefix = 'auth/'
            # ...
    
    The template name is assembled from the template path prefix, the
    re-formatted resource name, and the current action name. So, for a `User`
    resource, with `template_path_prefix = 'auth/'`, and an action of `show`,
    the template name would be:
    
        auth/user/show.html
    
    Finally, this is rendered using `render_to_response()`. The resource is
    passed into the context as `self`, so that attribute assignments from the
    action will be available in the template. This also uses `RequestContext`,
    so configured context processors will also be available.
    """
    
    from django.shortcuts import render_to_response
    from django.template import RequestContext
    
    resource_label = camel_to_underscore(resource_name(resource))
    template_path_prefix = getattr(resource, 'template_path_prefix', "")
    template_name = "%s%s/%s.html" % (template_path_prefix, resource_label, action.name)
    
    return render_to_response(template_name, {
        'self': resource
    }, context_instance=RequestContext(resource.request))



# -*- coding: utf-8 -*-

from django.http import Http404, HttpResponseNotAllowed
from djclsview import View

__all__ = ['Resource']


METHOD_ACTION_MAP = {
    'singular': {
        'show': {
            'GET': 'show',
            'POST': 'create',
            'PUT': 'update',
            'DELETE': 'destroy',
        },
        'new': {'GET': 'new'},
        'edit': {'GET': 'edit'},
    },
    
    'plural': {
        'index': {
            'GET': 'index',
            'POST': 'create'
        },
        'show': {
            'GET': 'show',
            'PUT': 'update',
            'POST': 'update',
            'DELETE': 'destroy',
        },
        'new': {'GET': 'new'},
        'edit': {'GET': 'edit'},
    }
}


class Resource(View):
    
    def __init__(self, request, *args, **params):
        self.request = request
        self.args = args
        self.params = params
    
    def __call__(self):
        """Dispatch to an action based on HTTP method + URL."""
        
        method = self.request.POST.get('_method', self.request.method).upper()
        # The action to use if it were a GET request.
        get_action = self.params.pop('action')
        
        try:
            # Singular or plural?
            mode = self.params.pop('mode')
            # The action to use for this particular request method.
            method_action = METHOD_ACTION_MAP[mode][get_action][method]
        except KeyError:
            # Assume the developer knows what he/she is doing.
            method_action = get_action
        
        if hasattr(self, method_action):
            return getattr(self, method_action)()
        elif hasattr(self, get_action):
            allowed_methods = []
            for meth, action_name in METHOD_ACTION_MAP[get_action].items():
                if hasattr(self, action_name):
                    allowed_methods.append(meth)
            return HttpResponseNotAllowed(allowed_methods)
        else:
            raise Http404
    
    def _format(self):
        """Return a mimetype shortcode, in case there's no Accept header."""
        
        return self.request.GET.get('format')



# -*- coding: utf-8 -*-

from django.conf.urls.defaults import include, patterns, url

__all__ = ['resources', 'resource']


def resources(resource_name, id=r'\d+', actions=None, name=None):
    
    """
    Helper for connecting to resource collections from your URLconf.
    
    Basic Usage:
    
        from django.conf.urls.defaults import *
        from dagny.urls import resources
        
        urlpatterns = patterns('',
            (r'^users/', resources('myapp.resources.User', name='User'))
        )
    
    Remember not to terminate the regex with a `$` character; `resources()`
    actually builds and `include()`s a sub-URLconf, and terminating the regex
    will prevent it from matching anything but the index.
    
    You also need to pass in the full identifier of your `Resource`, even if you
    specified a common prefix in the call to `patterns()`.
    
    The `name` parameter affects how you’ll do URL reversing; see the section
    below on reversing for in-depth information.
    
    `resources()` generates 4 basic types of URI, and 3 aliases:
    
        Path                Action  Aliases
        -------------------------------------------
        /items/             index   create
        /items/new/         new
        /items/<id>/        show    update, destroy
        /items/<id>/edit/   edit
    
    Note that if a request is received for an undefined action, your `Resource`
    will return the appropriate 404 or 405 response, regardless of what is
    defined in your URLconf.
    
    You can restrict the generated patterns by passing the `actions` keyword
    argument to `resources()`:
    
        urlpatterns = patterns('',
            (r'^items/', resources('myapp.resources.Item', actions=('index', 'show')))
        )
    
    The `<id>` parameter mentioned above can also be customized; by default it
    is just a regex pattern which looks for one or more digits (`r'\d+'`), but
    you could use slugs instead by passing an appropriate `id` keyword argument:
    
        urlpatterns = patterns('',
            (r'^items/', resources('myapp.resources.Item', id=r'[\w\-_]+'))
        )
    
    This is where the `actions` parameter becomes useful; if you're using slugs,
    you might want to use a different `new` URI, otherwise you'd need to check
    that no object had a slug of `'new'`.
    
    
    ## Reversing
    
    `resources()` names all of the URLs it generates, which makes it easy for
    you to use Django's existing helpers and templatetags for forms and links.
    By default, the names are just the full identifier of your `Resource`, and
    the particular action, separated by a hash. For example:
    
        class User(models.Model):
            # ...
            
            @models.permalink
            def get_absolute_url(self):
                return ('myapp.resources.User#show', self.id)
    
    Or in templates:
    
        <a href="{% url myapp.resources.User#new %}">Sign Up</a>
        
        <form method="post" action="{% url myapp.resources.User#create %}">
            ...
        </form>
        
        <a href="{% url myapp.resources.User#show user.id %}">View User</a>
        
        <form method="post" action="{% url myapp.resources.User#update user.id %}">
            ...
        </form>
    
    If you want to use shorter identifiers, just pass the `name` keyword
    argument to `resources()`:
    
        urlpatterns = patterns('',
            (r'^users/', resources('myapp.resources.User', name='User'))
        )
    
    And now you can just use that as the prefix:
    
        <a href="{% url User#new %}">Sign Up</a>
    
    """
    
    action_patterns = [
        # (action_name, pattern, action_param)
        ('index',   r'^$',                      'index'),
        ('create',  r'^$',                      'index'),
        ('new',     r'^new/$',                  'new'),
        ('show',    r'^(' + id + r')/$',        'show'),
        ('update',  r'^(' + id + r')/$',        'show'),
        ('destroy', r'^(' + id + r')/$',        'show'),
        ('edit',    r'^(' + id + r')/edit/$',   'edit'),
    ]
        
    if actions is None:
        actions = ('index', 'create', 'new', 'show', 'update', 'destroy', 'edit')
    
    if name is None:
        name = resource_name
    
    args = []
    
    for action_name, pattern, action_param in action_patterns:
        if action_name in actions:
            args.append(url(pattern, resource_name,
                            kwargs={'action': action_param, 'mode': 'plural'},
                            name=("%s#%s" % (name, action_name))))
    
    return include(patterns('', *args))


def resource(resource_name, actions=None, name=None):
    
    """
    Helper for connecting to singular resources from your URLconf.
    
    Usage is very similar to `resources()`:
    
        from django.conf.urls.defaults import *
        from dagny.urls import resource
        
        urlpatterns = patterns('',
            (r'^account/', resource('myapp.resources.User'))
        )
    
    `resource()` generates 3 types of URI, and 3 aliases:
    
        Path              Action  Aliases
        -------------------------------------------------
        /account/         show    create, update, destroy
        /account/new/     new
        /account/edit/    edit
    
    As with `resources()`, you can pass the `actions` keyword argument to
    restrict the generated URIs:
    
        urlpatterns = patterns('',
            (r'^account/', resource('myapp.resources.User', actions=('show',)))
        )
    
    The `name` parameter works in the same way as with `resources()`:
    
        # urls.py:
        
        urlpatterns = patterns('',
            (r'^account/', resource('myapp.resources.User', name="Account"))
        )
        
        # account/show.html
        
        <a href="{% url Account#edit %}">Edit Account</a>
    
    """
    
    action_patterns = [
        # (action_name, pattern, action_param)
        ('show',    r'^$',      'show'),
        ('create',  r'^$',      'show'),
        ('update',  r'^$',      'show'),
        ('destroy', r'^$',      'show'),
        ('new',     r'^new/$',  'new'),
        ('edit',    r'^edit/$', 'edit'),
    ]
        
    if actions is None:
        actions = ('show', 'create', 'update', 'destroy', 'new', 'edit')
    
    if name is None:
        name = resource_name
    
    args = []
    
    for action_name, pattern, action_param in action_patterns:
        if action_name in actions:
            args.append(url(pattern, resource_name,
                            kwargs={'action': action_param, 'mode': 'singular'},
                            name=("%s#%s" % (name, action_name))))
    
    return include(patterns('', *args))



# -*- coding: utf-8 -*-

import re


def camel_to_underscore(camel_string):
    
    """
    Convert a CamelCase string to under_score.
    
    Examples:
    
        >>> camel_to_underscore('SplitAtTheBoundaries')
        'split_at_the_boundaries'
        
        >>> camel_to_underscore('XYZResource')
        'xyz_resource'
        
        >>> camel_to_underscore('ResourceXYZ')
        'resource_xyz'
        
        >>> camel_to_underscore('XYZ')
        'xyz'
    
    """
    
    return re.sub(
        r'(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))', r'_\1',
        camel_string).lower().strip('_')


def resource_name(resource_cls):
    """Return the name of a given resource, stripping 'Resource' off the end."""
    
    from dagny import Resource
    
    if isinstance(resource_cls, Resource):
        resource_cls = resource_cls.__class__
    
    name = resource_cls.__name__
    if name.endswith('Resource'):
        return name[:-8]
    return name



# -*- coding: utf-8 -*-

from functools import wraps

import odict

from dagny import conneg


class Skip(Exception):
    
    """
    Move on to the next renderer backend.
    
    This exception can be raised by a renderer backend to instruct the
    `Renderer` to ignore the current backend and move on to the next-best one.
    """


class Renderer(object):
    
    """
    Manage a collection of renderer backends, and their execution on an action.
    
    A renderer backend is a callable which accepts an `Action` and a `Resource`
    and returns an instance of `django.http.HttpResponse`. For example:
    
        >>> def render_html(action, resource):
        ...     from django.http import HttpResponse
        ...     return HttpResponse(content="<html>...</html>")
    
    Backends are associated with mimetypes on the `Renderer`, through mimetype
    shortcodes (see `dagny.conneg` for more information on shortcodes). The
    `Renderer` exports a dictionary-like interface for managing these
    associations:
    
        >>> r = Renderer()
        
        >>> r['html'] = render_html
        
        >>> r['html']  # doctest: +ELLIPSIS
        <function render_html at 0x...>
        
        >>> 'html' in r
        True
        
        >>> del r['html']
        
        >>> r['html']
        Traceback (most recent call last):
            ...
        KeyError: 'html'
        
        >>> 'html' in r
        False
    
    A few helpful dictionary methods have also been added, albeit
    underscore-prefixed to prevent naming clashes. Behind the scenes, `Renderer`
    uses [odict](http://pypi.python.org/pypi/odict), which will keep the keys in
    the order they were *first* defined. Here are a few examples:
    
        >>> r['html'] = 1
        >>> r['json'] = 2
        >>> r['xml'] = 3
        
        >>> r._keys()
        ['html', 'json', 'xml']
        
        >>> r._items()
        [('html', 1), ('json', 2), ('xml', 3)]
        
        >>> r._values()
        [1, 2, 3]
    
    This order preservation is useful for ConNeg, since you can define backends
    in order of server preference and the negotiator will consider them
    appropriately. You can push something to the end of the queue by removing
    and then re-adding it:
    
        >>> r['html'] = r._pop('html')
        
        >>> r._keys()
        ['json', 'xml', 'html']
    
    You can also define backends using a handy decorator-based syntax:
    
        >>> @r.html
        ... def render_html_2(action, resource):
        ...     from django.http import HttpResponse
        ...     return HttpResponse(content="<html>...</html>")
        
        >>> r['html'] is render_html_2
        True
    
    Remember that your shortcode *must* be pre-registered with
    `dagny.conneg.MIMETYPES` for this to work, otherwise an `AttributeError`
    will be raised. This also introduces the constraint that your shortcode must
    be a valid Python identifier.
    """
    
    def __init__(self, backends=None):
        if backends is None:
            backends = odict.odict()
        else:
            backends = backends.copy()
        self._backends = backends
    
    def __getattr__(self, shortcode):
        
        """
        Support use of decorator syntax to define new renderer backends.
        
            >>> r = Renderer()
            
            >>> @r.html
            ... def render_html(action, resource):
            ...     return "<html>...</html>"
            
            >>> render_html  # doctest: +ELLIPSIS
            <function render_html at 0x...>
            
            >>> r['html']  # doctest: +ELLIPSIS
            <function render_html at 0x...>
            
            >>> r['html'] is render_html
            True
        
        """
        
        if shortcode not in conneg.MIMETYPES:
            raise AttributeError(shortcode)
        
        def decorate(function):
            self[shortcode] = function
            return function
        return decorate
    
    def __call__(self, action, resource):
        matches = self._match(action, resource)
        
        for shortcode in matches:
            try:
                return self[shortcode](action, resource)
            except Skip:
                continue
        
        return not_acceptable(action, resource)
    
    def _match(self, action, resource):
        """Return all matching shortcodes for a given action and resource."""
        
        matches = []
        
        format_override = resource._format()
        if format_override and (format_override in self._keys()):
            matches.append(format_override)
        
        accept_header = resource.request.META.get('HTTP_ACCEPT')
        if accept_header:
            matches.extend(conneg.match_accept(accept_header, self._keys()))
        
        if (not matches) and ('html' in self):
            matches.append('html')
        
        return matches
    
    def _bind(self, action):
        
        """
        Bind this `Renderer` to an action, returning a `BoundRenderer`.
        
            >>> r = Renderer()
            >>> action = object()
            >>> r['html'] = 1
            
            >>> br = r._bind(action)
            >>> br  # doctest: +ELLIPSIS
            <BoundRenderer on <object object at 0x...>>
        
        Associations should be preserved, albeit on a copied `odict`, so that
        modifications to the `BoundRenderer` do not propagate back to this.
        
            >>> br['html']
            1
            >>> br['html'] = 2
            >>> br['html']
            2
            >>> r['html']
            1
            >>> r['html'] = 3
            >>> r['html']
            3
            >>> br['html']
            2
        
        """
        
        return BoundRenderer(action, backends=self._backends)
    
    def _copy(self):
        return type(self)(backends=self._backends)
    
    ### <meta>
    #
    #   This chunk of code creates several proxy methods going through to
    #   `_backends`. A group of them are underscore-prefixed to prevent naming
    #   clashes with the `__getattr__`-based decorator syntax (so you could
    #   still associate a backend with a shortcode of 'pop', for example).
    
    proxy = lambda meth: property(lambda self: getattr(self._backends, meth))
    
    for method in ('__contains__', '__getitem__', '__setitem__', '__delitem__'):
        vars()[method] = proxy(method)
    
    for method in ('clear', 'get', 'items', 'iteritems', 'iterkeys',
                   'itervalues', 'keys', 'pop', 'popitem', 'ritems',
                   'riteritems', 'riterkeys', 'ritervalues', 'rkeys', 'rvalues',
                   'setdefault', 'sort', 'update', 'values'):
        vars()['_' + method] = proxy(method)
    
    _dict = proxy('as_dict')
    
    del method, proxy
    
    #
    ### </meta>


class BoundRenderer(Renderer):
    
    def __init__(self, action, backends=None):
        super(BoundRenderer, self).__init__(backends=backends)
        self._action = action
    
    def __repr__(self):
        return "<BoundRenderer on %r>" % (self._action,)
    
    def __getattr__(self, shortcode):
        
        """
        Support use of decorator syntax to define new renderer backends.
        
        In this case, decorated functions should be methods which operate on a
        resource, and take no other arguments.
        
            >>> action = object()
            >>> r = BoundRenderer(action)
            >>> old_action_id = id(action)
            
            >>> @r.html
            ... def action(resource):
            ...     return "<html>...</html>"
            
            >>> id(action) == old_action_id # Object has not changed.
            True
        
        Functions will be wrapped internally, so that their function signature
        is that of a generic renderer backend. Accessing the 
        
            >>> resource = object()
            >>> r['html'](action, resource)
            '<html>...</html>'
        
        """
        
        if shortcode not in conneg.MIMETYPES:
            raise AttributeError(shortcode)
        
        def decorate(method):
            self[shortcode] = resource_method_wrapper(method)
            return self._action
        return decorate
    
    def __call__(self, resource):
        return super(BoundRenderer, self).__call__(self._action, resource)


def resource_method_wrapper(method):
    
    """
    Wrap a 0-ary resource method as a generic renderer backend.
    
        >>> @resource_method_wrapper
        ... def func(resource):
        ...     print repr(resource)
        
        >>> action = "abc"
        >>> resource = "def"
        
        >>> func(action, resource)
        'def'
    
    """
    
    def generic_renderer_backend(action, resource):
        return method(resource)
    return generic_renderer_backend


def not_acceptable(action, resource):
    """Respond, indicating that no acceptable entity could be generated."""
    
    from django.http import HttpResponse
    
    response = HttpResponse(status=406)  # Not Acceptable
    del response['Content-Type']
    return response



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import glob
import os
import re

from distribute_setup import use_setuptools; use_setuptools()
from setuptools import setup, find_packages


rel_file = lambda *args: os.path.join(os.path.dirname(os.path.abspath(__file__)), *args)

def read_from(filename):
    fp = open(filename)
    try:
        return fp.read()
    finally:
        fp.close()

def lines_from(filename):
    return filter(None, map(lambda s: s.strip(), read_from(filename).splitlines()))

def get_version():
    data = read_from(rel_file('src', 'dagny', '__init__.py'))
    return re.search(r"__version__ = '([^']+)'", data).group(1)

def get_requirements():
    return lines_from(rel_file('REQUIREMENTS'))

def get_extra_requirements():
    extras_require = {}
    for req_filename in glob.glob(rel_file('REQUIREMENTS.*')):
        group = os.path.basename(req_filename).split('.', 1)[1]
        extras_require[group] = lines_from(req_filename)
    return extras_require


setup(
    name             = 'dagny',
    version          = get_version(),
    author           = "Zachary Voase",
    author_email     = "z@zacharyvoase.com",
    url              = 'http://github.com/zacharyvoase/dagny',
    description      = "Rails-style Resource-Oriented Architecture for Django.",
    packages         = find_packages(where='src'),
    package_dir      = {'': 'src'},
    install_requires = get_requirements(),
    extras_require   = get_extra_requirements(),
)



# -*- coding: utf-8 -*-

"""
Helpers and global mappings for content negotiation.

If you want to define a custom mimetype shortcode, add it to the `MIMETYPES`
dictionary in this module (without the leading '.' character). For example:

    from dagny.conneg import MIMETYPES
    
    MIMETYPES['png'] = 'image/png'
    MIMETYPES['json'] = 'text/javascript'

"""

import mimetypes

from webob.acceptparse import MIMEAccept

__all__ = ['MIMETYPES', 'match_accept']


# Maps renderer shortcodes => mimetypes.
MIMETYPES = {
    'rss': 'application/rss+xml',
    'json': 'application/json',
    'rdf_xml': 'application/rdf+xml',
    'xhtml': 'application/xhtml+xml',
    'xml': 'application/xml',
}


# Load all extension => mimetype mappings from `mimetypes` stdlib module.
for ext, mimetype in mimetypes.types_map.iteritems():
    shortcode = ext.lstrip(".").replace(".", "_")  # .tar.bz2 => tar_bz2
    MIMETYPES.setdefault(shortcode, mimetype)
del ext, shortcode, mimetype  # Clean up


def match_accept(header, shortcodes):
    
    """
    Match an Accept header against a list of shortcodes, in order of preference.
    
    A few examples:
    
        >>> header = "application/xml,application/xhtml+xml,text/html"
        
        >>> match_accept(header, ['html', 'json', 'xml'])
        ['html', 'xml']
        
        >>> header2 = "application/json,application/xml"
        
        >>> match_accept(header2, ['html', 'json', 'xml'])
        ['json', 'xml']
        
        >>> match_accept(header2, ['html', 'xml', 'json'])
        ['xml', 'json']
    
    """
    
    server_types = map(MIMETYPES.__getitem__, shortcodes)
    client_types = MIMEAccept("Accept", header).best_matches()
    matches = []
    for mimetype in server_types:
        if mimetype in client_types:
            matches.append(mimetype)
    
    return map(shortcodes.__getitem__, map(server_types.index, matches))
